<?php
// delicious-daily-post-formatter.php
//
// The formatter of Del.icio.us Daily Posts.
//
// Copyright (C) 2006 Kevin Wetzels
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

@include_once(dirname(__FILE__) . '/delicious-daily-post-objects.php');

class DeliciousDailyPostFormatter {
	
	var $regular_post;
	
	var $variables;
	
	var $formatted;
	
	var $date_of_post;
	
	function build_variables($post) {
		$this->variables['count'] = count($post->bookmarks);
	}
	
	// Formats the given post according to the format specified by the user.
	function format($post, $formats, $limit) {
		$this->build_variables($post);
		$this->date_of_post = mktime (0, 0, 0, $post->month_of_year, $post->day_of_month, $post->year);
		$this->regular_post = new DeliciousDailyPostRegularWordPressPost();
		$this->regular_post->title = $this->format_title($formats);
		
		$before_content = $this->replace_date_variables($formats['before_bookmarks']);
		$before_content = $this->replace_variables($before_content);
		$this->regular_post->content = $before_content;
		
		$this->variables['nr'] = 0;
		foreach ($post->bookmarks as $bookmark) {
			if ($this->variables['nr'] > 0 && $this->variables['nr'] == $limit) {
				break;
			}			
			++$this->variables['nr'];
			$this->variables['extended'] = $bookmark->extended;
			$this->variables['title'] = $bookmark->title;
			$this->variables['url'] = $bookmark->url;
			$this->variables['tags'] = '';
			foreach ($bookmark->tags as $tag) {
				$this->variables['tag'] = $tag->tag;
				$this->variables['tag-url'] = $tag->tag_url;
				$tag_string = $this->replace_date_variables($formats['tag']);
				$this->variables['tags'] .= ' ' . $this->replace_tag_variables($tag_string);
			}		
			$temp = $this->replace_date_variables($formats['bookmark']);
			$this->regular_post->content .= $this->replace_variables($temp);
		}
		
		$after_content = $this->replace_date_variables($formats['after_bookmarks']);
		$after_content = $this->replace_variables($after_content);
		$this->regular_post->content .= $after_content;
		
		$this->variables['bookmarks'] = $this->regular_post->content;
		$this->regular_post->excerpt = $this->replace_date_variables($formats['excerpt']);
		$this->regular_post->excerpt = $this->replace_excerpt_variables($this->regular_post->excerpt);
			
		return $this->regular_post;
	}
	
	// Format the title, which can only contain date variables and a count.
	function format_title($formats) {
		$title = $this->replace_date_variables($formats['title']);
		$title = $this->replace_variables($title);
		return $title;
	}
	
	// Replace the tag specific variables.
	function replace_tag_variables($string) {
		$pattern = "/(?s)(.*)(%(tag|tag-url)%)(.*)/";
		while (preg_match($pattern, $string)) {
			$string = preg_replace_callback($pattern, array(&$this, 'replace_variable'), $string);
		}
		return $string;
	}
	
	// Replace the excerpt specific variables.
	function replace_excerpt_variables($string) {
		$pattern = "/(?s)(.*)(%(count|bookmarks)%)(.*)/";
		while (preg_match($pattern, $string)) {
			$string = preg_replace_callback($pattern, array(&$this, 'replace_variable'), $string);
		}
		return $string;
	}
	
	// Replace the variables for a bookmark.
	function replace_variables($string) {
		$pattern = "/(?s)(.*)(%(count|url|title|extended|tags|nr)%)(.*)/";
		while (preg_match($pattern, $string)) {
			$string = preg_replace_callback($pattern, array(&$this, 'replace_variable'), $string);
		}
		return $string;
	}
	
	// Replace a variable.
	function replace_variable($matches) {
		return $matches[1] . $this->variables[$matches[3]] . (isset($matches[4]) ? $matches[4] : '');
	}
	
	// Replace date variables.
	function replace_date_variables($string) {
		$pattern = "/(?s)(.*)(%(Z|z|y|Y|W|w|U|T|t|S|s|r|O|n|M|m|L|l|j|I|i|H|h|G|g|F|D|d|B|A|a)%)(.*)/";
		while (preg_match($pattern, $string)) {
			$string = preg_replace_callback($pattern, array(&$this, 'replace_date_variable'), $string);
		}
		return $string;
	}
	
	// Replace a date variable.
	function replace_date_variable($matches) {	
		return $matches[1] . __(date($matches[3], $this->date_of_post)) . (isset($matches[4]) ? $matches[4] : '');
	}
	
}

?>