<?php
// delicious-daily-post-parser.php
//
// The parser of Del.icio.us Daily Posts.
//
// Copyright (C) 2006 Kevin Wetzels
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

@include_once(dirname(__FILE__) . '/delicious-daily-post-objects.php');

class DeliciousDailyPostParser {
	
	var $post;
	var $current_bookmark;
	
	// Checks whether the post is a del.icio.us daily post by checking whether
	// the title of the post adheres to the del.icio.us daily post title format
	// of "links for Y-m-d".
	function isDeliciousDailyPost($title) {
		$pattern = "/links for \d{4}-\d{2}-\d{2}/";
		return preg_match($pattern, $title);
	}
	
	// Parses the contents of the del.icio.us daily post. Returns a 
	// DeliciousDailyPost object.
	function parse($title, $content) {
		
		$this->post = new DeliciousDailyPost();		
		list($this->post->year, $this->post->month_of_year, $this->post->day_of_month) = sscanf($title, "links for %d-%d-%d");
		
		$pattern = "/(?s)(.*)\\<li\\>(.*)\\<\\/li\\>(.*)/";
		while (preg_match($pattern, $content)) {
			$this->current_bookmark = new DeliciousDailyPostBookmark();
			$content = preg_replace_callback($pattern, array(&$this, 'parse_bookmark'), $content);		
			$this->post->bookmarks[] = $this->current_bookmark;
		}
		return $this->post;
	}
	
	// Parses a del.icio.us bookmark. This is the stuff inside the <li></li> tags.
	function parse_bookmark($bookmark_matches) {
		$bookmark_string = $bookmark_matches[2];
		$pattern = "/(?s)(.*)\\<div([^\\>]*)\\>(.*)\\<\\/div\\>(.*)/";
		if (preg_match($pattern, $bookmark_string)) {
			$bookmark_string = preg_replace_callback($pattern, array(&$this, 'parse_tags'), $bookmark_string);
		}
		$nrDivs = substr_count($bookmark_string, "<div");
		if ($nrDivs > 1 && preg_match($pattern, $bookmark_string)) {
			$bookmark_string = preg_replace_callback($pattern, array(&$this, 'parse_extended'), $bookmark_string);
		}
		if (preg_match($pattern, $bookmark_string)) {
			$bookmark_string = preg_replace_callback($pattern, array(&$this, 'parse_title'), $bookmark_string);
		}
		return $bookmark_matches[1] . (isset($title_matches[4]) ? $title_matches[4] : '');
	}
	
	// Parses the title of del.icio.us bookmark.
	function parse_title($title_matches) {
		$title_string = $title_matches[3];
		$pattern = "/(?s)(.*)\\<a(.*)href=\"(.*)\"([^\\>]*)\\>(.*)\\<\\/a\\>(.*)/";
		while (preg_match($pattern, $title_string)) {
			$title_string = preg_replace_callback($pattern, array(&$this, 'extract_title'), $title_string);
		}
		return $title_matches[1] . (isset($title_matches[4]) ? $title_matches[4] : '');
	}
	
	// Extracts the title information.
	function extract_title($matches) {
		$this->current_bookmark->url = $matches[3];
		$this->current_bookmark->title = $matches[5];
		return $matches[1] . (isset($matches[6]) ? $matches[6] : '');
	}
	
	// Parses the extended part of the bookmark (a description of the bookmark).
	function parse_extended($extended_matches) {
		$this->current_bookmark->extended = $extended_matches[3];
		return $extended_matches[1] . (isset($extended_matches[4]) ? $extended_matches[4] : '');
	}
	
	// Parses the tags of a bookmark.
	function parse_tags($tag_matches) {
		$tags_string = $tag_matches[3];
		$pattern = "/(?s)(.*)\\<a(.*)href=\"(.*)\"([^\\>]*)\\>(.*)\\<\\/a\\>(.*)/";
		while (preg_match($pattern, $tags_string)) {		
			$tags_string = preg_replace_callback($pattern, array(&$this, 'extract_tag'), $tags_string);
		}
		return $tag_matches[1] . (isset($tag_matches[4]) ? $tag_matches[4] : '');
	}
	
	// Extracts the tag information.
	function extract_tag($matches) {
		$tag = new DeliciousDailyPostTag();
		$tag->tag_url = $matches[3];
		$tag->tag = $matches[5];
		$this->current_bookmark->tags[] = $tag;
		return $matches[1] . (isset($matches[6]) ? $matches[6] : '');
	}
	
}

?>